@@ -43,5 +43,27 @@ def evaluateTokens(requestContext, tokens):
     return tokens.boolean[0] == 'true'
 
 
+def extractPathExpressions(targets):
+  # Returns a list of unique pathExpressions found in the targets list
+
+  pathExpressions = []
+
+  def extractPathExpression(tokens):
+    if tokens.expression:
+      return extractPathExpression(tokens.expression)
+    elif tokens.pathExpression:
+      pathExpressions.append(tokens.pathExpression)
+    elif tokens.call:
+      [extractPathExpression(arg) for arg in tokens.call.args]
+
+  for target in targets:
+    tokens = grammar.parseString(target)
+    extractPathExpression(tokens)
+
+  s = set(pathExpressions)
+  pathExpressions = list(s)
+  return pathExpressions
+
+
 #Avoid import circularities
 from graphite.render.functions import SeriesFunctions
